﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace Bird_tool
{
    public partial class ProgressPanel : UserControl
    {
        // 添加计时器和开始时间字段
        private System.Windows.Forms.Timer _timer;
        private DateTime _startTime;
        private Label _totalTimeLabel; // 总耗时标签



        public ProgressPanel()
        {
            InitializeTimer();
            SetupUI();
        }

        private void InitializeTimer()
        {
            _timer = new System.Windows.Forms.Timer();
            _timer.Interval = 1000; // 每秒更新一次
            _timer.Tick += Timer_Tick;
        }

        private void SafeInvoke(Action action)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(action);
            }
            else
            {
                action();
            }
        }

        // 步骤项数据结构
        public class TestStep
        {
            public string Name { get; set; }
            public StepStatus Status { get; set; }
            public string Details { get; set; }
            public DateTime? StartTime { get; set; }
            public DateTime? EndTime { get; set; }
            public TimeSpan? Duration => EndTime.HasValue && StartTime.HasValue
                ? EndTime - StartTime
                : null;
        }

        

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (_startTime != DateTime.MinValue)
            {
                TimeSpan elapsed = DateTime.Now - _startTime;
                _totalTimeLabel.Text = $"总耗时: {elapsed:hh\\:mm\\:ss}";
            }
        }

        private void SetupUI()
        {
            // 主布局
            this.BackColor = SystemColors.Window;
            this.Dock = DockStyle.Fill;
            this.Padding = new Padding(20);

            // 主表格布局
            TableLayoutPanel mainLayout = new TableLayoutPanel
            {
                Dock = DockStyle.Fill,
                ColumnCount = 2, // 改为两列
                RowCount = 6,
                ColumnStyles =
                {
                    new ColumnStyle(SizeType.Percent, 30F), // 左侧列 (步骤列表)
                    new ColumnStyle(SizeType.Percent, 70F)  // 右侧列 (信息区域)
                },
                RowStyles =
                {
                    new RowStyle(SizeType.AutoSize),
                    new RowStyle(SizeType.AutoSize),
                    new RowStyle(SizeType.AutoSize),
                    new RowStyle(SizeType.AutoSize),
                    new RowStyle(SizeType.Percent, 100F), // 主内容区域
                    new RowStyle(SizeType.AutoSize)
                }
            };

            // 1. 进度条
            ProgressBar = new ProgressBar
            {
                Dock = DockStyle.Fill,
                Height = 30,
                Style = ProgressBarStyle.Continuous,
                Margin = new Padding(0, 0, 0, 15)
            };

            // 2. 当前测试项标签
            CurrentTestLabel = new Label
            {
                Dock = DockStyle.Fill,
                Text = "当前测试项：初始化中...",
                Font = new Font("Microsoft YaHei UI", 20F, FontStyle.Bold),
                TextAlign = ContentAlignment.MiddleLeft,
                Margin = new Padding(0, 0, 0, 20),
                AutoSize = true
            };
            // 测试步骤标签
            StepLabel = new Label
            {
                Dock = DockStyle.Fill,
                Text = "状态: 准备开始",
                Font = new Font("Microsoft YaHei UI", 9F, FontStyle.Regular),
                TextAlign = ContentAlignment.MiddleLeft,
                ForeColor = Color.DarkGray,
                Margin = new Padding(0, 0, 0, 5),
                AutoSize = true
            };
            // 4. 新增：总耗时标签
            _totalTimeLabel = new Label
            {
                Dock = DockStyle.Fill,
                Text = "总耗时: 00:00:00",
                Font = new Font("Microsoft YaHei UI", 9F, FontStyle.Regular),
                TextAlign = ContentAlignment.MiddleRight, // 靠右显示
                ForeColor = Color.DarkBlue,
                Margin = new Padding(0, 0, 0, 10),
                AutoSize = true
            };

            // 创建测试步骤列表
            StepList = new TestStepList();

            // 中央提示区域
            Panel centerPanel = new Panel
            {
                Dock = DockStyle.Fill,
                BackColor = Color.Transparent
            };

            // 提示文字（居中）
            MessageLabel = new Label
            {
                Text = "测试进行中，请稍候...",
                Font = new Font("Microsoft YaHei UI", 24F, FontStyle.Regular),
                TextAlign = ContentAlignment.MiddleCenter,
                Dock = DockStyle.Fill,
                AutoSize = false
            };

            // 图片显示区域
            ImageBox = new PictureBox
            {
                SizeMode = PictureBoxSizeMode.Zoom,
                Visible = false,
                Dock = DockStyle.Top,
                Height = 150
            };

            // 组装中央面板
            centerPanel.Controls.Add(MessageLabel);
            centerPanel.Controls.Add(ImageBox);

            // 4. 日志按钮区域
            // 5. 按钮区域
            FlowLayoutPanel buttonPanel = new FlowLayoutPanel
            {
                Dock = DockStyle.Fill,
                AutoSize = true,
                FlowDirection = FlowDirection.RightToLeft,
                Padding = new Padding(5),
                Margin = new Padding(0, 20, 0, 0)
            };

            LogButton = new Button
            {
                Text = "显示日志",
                Size = new Size(100, 50),
                Font = new Font("Microsoft YaHei UI", 14F),
                BackColor = Color.LightGray,
                FlatStyle = FlatStyle.Flat,
                Margin = new Padding(5)
            };
            LogButton.Click += LogButton_Click;

            // 停止按钮
            StopButton = new Button
            {
                Text = "停止测试",
                Size = new Size(100, 50),
                Font = new Font("Microsoft YaHei UI", 14F),
                BackColor = Color.LightCoral,
                FlatStyle = FlatStyle.Flat,
                Margin = new Padding(5),
                Tag = "stop" // 用于标识按钮状态
            };
            StopButton.Click += StopButton_Click;

            StartButton = new Button
            { 
                Text = "开始测试",
                Size = new Size(100, 50),
                Font = new Font("Microsoft YaHei UI", 14F),
                BackColor = Color.Green,
                FlatStyle = FlatStyle.Flat,
                Margin = new Padding(5),
                Visible = false
            };
            StartButton.Click += StartButton_Click;

            // 组装面板
            buttonPanel.Controls.Add(LogButton);
            buttonPanel.Controls.Add(StartButton);
            buttonPanel.Controls.Add(StopButton);

            mainLayout.Controls.Add(ProgressBar, 0, 0);
            mainLayout.SetColumnSpan(ProgressBar, 2); // 跨两列

            mainLayout.Controls.Add(CurrentTestLabel, 0, 1);
            mainLayout.SetColumnSpan(CurrentTestLabel, 2); // 跨两列

            mainLayout.Controls.Add(StepLabel, 0, 2);
            mainLayout.SetColumnSpan(StepLabel, 2); // 跨两列

            // 添加总耗时标签（跨两列）
            mainLayout.Controls.Add(_totalTimeLabel, 0, 3);
            mainLayout.SetColumnSpan(_totalTimeLabel, 2);

            mainLayout.Controls.Add(StepList, 0, 4); // 新增步骤列表
            mainLayout.Controls.Add(centerPanel, 1, 4);

            mainLayout.Controls.Add(buttonPanel, 0, 5);
            mainLayout.SetColumnSpan(buttonPanel, 2); // 跨两列

            this.Controls.Add(mainLayout);
        }

        // 公共属性
        public ProgressBar ProgressBar { get; private set; }
        public Label CurrentTestLabel { get; private set; }
        public Label StepLabel { get; private set; }
        public TestStepList StepList { get; private set; }
        public Label MessageLabel { get; private set; }
        public PictureBox ImageBox { get; private set; }
        public Button LogButton { get; private set; }
        public Button StopButton { get; private set; }
        public Button StartButton { get; private set; }
        private void LogButton_Click(object sender, EventArgs e)
        {
            OnLogRequested?.Invoke(this, EventArgs.Empty);
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            if (StopButton.Tag.ToString() == "stop")
            {
                // 停止测试
                OnStopRequested?.Invoke(this, EventArgs.Empty);
            }
            else
            {
                // 返回主界面
                OnReturnRequested?.Invoke(this, EventArgs.Empty);
            }
        }
        private void StartButton_Click(object sender, EventArgs e)
        {
            OnStartRequested?.Invoke(this, EventArgs.Empty);
            startTest();
        }
        public void startTest()
        {
            // 隐藏开始测试按钮
            StartButton.Visible = false;

            // 重置停止按钮
            StopButton.Text = "停止测试";
            StopButton.BackColor = Color.LightCoral;
            StopButton.Tag = "stop";
            StartTimer();
            StepList.SetTestRunning(true);

        }

        // 启动计时器
        public void StartTimer()
        {
            _startTime = DateTime.Now;
            _timer.Start();
        }

        // 停止计时器
        public void StopTimer()
        {
            _timer.Stop();
            // 更新最后一次时间
            if (_startTime != DateTime.MinValue)
            {
                TimeSpan elapsed = DateTime.Now - _startTime;
                _totalTimeLabel.Text = $"总耗时: {elapsed:hh\\:mm\\:ss}";
            }
        }

        

        // 进度管理
        public int ProgressValue
        {
            get => ProgressBar.Value;
            set => ProgressBar.Value = Math.Max(0, Math.Min(100, value));
        }

        // 当前测试项
        public string CurrentTest
        {
            get => CurrentTestLabel.Text.Replace("当前测试项：", "");
            set => CurrentTestLabel.Text = $"当前测试项：{value}";
        }


        public void SetTestStatus(string statusText, Color color)
        {
            StepLabel.Text = statusText;
            StepLabel.ForeColor = color;
        }

        // 提示消息
        public string Message
        {
            get => MessageLabel.Text;
            set => MessageLabel.Text = value;
        }

        // 提示信息
        public string StateText
        {
            get => StepLabel.Text;
            set => StepLabel.Text = value;
        }

        // 设置图片
        public void SetImage(Image image)
        {
            if (image != null)
            {
                ImageBox.Image = image;
                ImageBox.Visible = true;
            }
            else
            {
                ImageBox.Visible = false;
            }
        }

        // 添加步骤的公共方法
        public void AddTestStep(string key, string name, string details = "")
        {
            StepList.AddStep(key, name);
        }



        // 更新步骤状态
        public void UpdateTestStepStatus(string key, string name, StepStatus status, string details = "")
        {
            StepList.UpdateStepStatus(key, name, status);
        }



        public void ShowTestResult(bool isSuccess, string message, Image resultImage = null)
        {
            SafeInvoke(() =>
            {
                try
                {
                    // 停止计时器
                    StopTimer();
                    StepList.SetTestRunning(false);
                    // 更新消息
                    MessageLabel.Text = message;
                    MessageLabel.ForeColor = isSuccess ? Color.Green : Color.Red;

                    // 设置结果图片
                    SetImage(resultImage);

                    // 更新停止按钮为返回按钮
                    StopButton.Text = "返回主页";
                    StopButton.BackColor = isSuccess ? Color.LightGreen : Color.LightPink;
                    StopButton.Tag = "return";

                    // 显示开始测试按钮
                    StartButton.Visible = true;

                    // 隐藏进度条
                    ProgressBar.Visible = false;

                    // 更新状态标签
                    StepLabel.Text = isSuccess ? "状态: 测试成功" : "状态: 测试失败";
                    StepLabel.ForeColor = isSuccess ? Color.Green : Color.Red;
                }
                catch (Exception ex)
                {
                    // 可以记录日志或显示错误
                    System.Diagnostics.Debug.WriteLine($"显示测试结果时出错: {ex.Message}");
                }
            });
        }

        // 测试面板重置

        // 重置测试面板
        public void ResetPanel()
        {
            // 重置计时器
            _timer.Stop();
            _totalTimeLabel.Text = "总耗时: 00:00:00";
            _startTime = DateTime.MinValue;

            ProgressBar.Visible = true;
            ProgressBar.Value = 0;
            CurrentTest = "准备开始";
            Message = "测试即将开始...";
            MessageLabel.ForeColor = SystemColors.ControlText;
            StepLabel.Text = "状态: 准备开始";
            StepLabel.ForeColor = Color.DarkGray;
            SetImage(null);

            // 重置停止按钮
            //StopButton.Text = "停止测试";
            //StopButton.BackColor = Color.LightCoral;
            //StopButton.Tag = "stop";

            StopButton.Text = "返回主页";
            StopButton.BackColor = Color.LightGreen;
            StopButton.Tag = "return";

            StartButton.Visible = true;

            StepList.Reset();
        }

        

        // 日志请求事件
        public event EventHandler OnLogRequested;
        public event EventHandler OnStopRequested;
        public event EventHandler OnReturnRequested;
        public event EventHandler OnStartRequested;
    }
}